%This m-file calculates the spot exchange rate at the next investigated time point
%It uses the m-files integral_1.m and integral_2.m
%If one wants to investigate another time point, integral_1.m and integral_2.m need to be adjusted
%The definite interval needs to be adjusted as well

%FIRST: Expectations at current time
clear all;

load ER_X_Y_at_t.mat
%Current exchange rate
ER_X_Y=ER_X_Y_at_t(3);
%Current time
t=ER_X_Y_at_t(1);
lasttime=ER_X_Y_at_t(2);


%COUNTRY X
%COUNTRY X with following values
load ValX.mat
r_X=ValX(1);
kappa=ValX(2);
theta=ValX(3);
sigma=ValX(4);
lambda=ValX(5);
gamma=((kappa+lambda)^2+2*(sigma^2))^0.5;
phi1=gamma;
phi2=(kappa+lambda+gamma)*0.5;
phi3=(2*kappa*theta)/(sigma^2);

%Calculation of the term structure of COUNTRY X
%Yield converges to the spot rate as maturity nears: Tau==Null
Rrt(1)=r_X;
%Value of Zero Bond at maturity and effective interest rate
Prt(1)=1;
i=1;    
for Tau=0.01:0.01:lasttime-t
    i=i+1;
    N=phi2*(exp(phi1*(Tau))-1)+phi1;    
    At=(phi1*exp(phi2*(Tau))/N)^phi3;
    Bt=(exp(phi1*(Tau))-1)/N;
    Prt(i)=At*exp(-Bt*r_X);
    if At~=0
        Rrt(i)=(-log(At)+r_X*Bt)/(Tau);
    else
        Rrt(i)=NaN;
    end
end

%Saving
ValuesX=[t:0.01:lasttime;Rrt;Prt]; 
datafile=['X',mat2str(r_X),'_',mat2str(kappa),'_',mat2str(theta),'_',mat2str(sigma),'_',mat2str(lambda),'.mat'];
save(datafile,'ValuesX');
phi_rX=[phi1,phi2,phi3,r_X,kappa,theta,sigma,lambda,gamma];
save X.mat phi_rX;

ff=t;
gg=r_X;

%Needed for SECOND part
phi1_1=gamma;
phi2_1=(kappa+lambda+gamma)*0.5;
phi3_1=(2*kappa*theta)/(sigma^2);
r1_1=r_X;
r2_1=input('Please enter the value (in %) of the spot rate in Country X at time t=0.01: ');
r2_1=r2_1/100;
r_X=r2_1;
ValX(1)=r_X;
save ValX.mat ValX

%COUNTRY Y
%COUNTRY Y with following values
load ValY.mat
r_Y=ValY(1);
kappa=ValY(2);
theta=ValY(3);
sigma=ValY(4);
lambda=ValY(5);
gamma=((kappa+lambda)^2+2*(sigma^2))^0.5;
phi1=gamma;
phi2=(kappa+lambda+gamma)*0.5;
phi3=(2*kappa*theta)/(sigma^2);

%Calculation of the term structure of COUNTRY Y
%Yield converges to the spot rate as maturity nears: Tau==Null
Rrt(1)=r_Y;
%Value of Zero Bond at maturity and effective yield
Prt(1)=1;
%i is used for counting within the array
i=1;    
for Tau=0.01:0.01:lasttime-t
    i=i+1;
    N=phi2*(exp(phi1*(Tau))-1)+phi1;    
    At=(phi1*exp(phi2*(Tau))/N)^phi3;
    Bt=(exp(phi1*(Tau))-1)/N;
    Prt(i)=At*exp(-Bt*r_Y);
    if At~=0
        Rrt(i)=(-log(At)+r_Y*Bt)/(Tau);
    else
        Rrt(i)=NaN;
    end
end

%Saving
ValuesY=[t:0.01:lasttime;Rrt;Prt]; 
datafile=['Y',mat2str(r_Y),'_',mat2str(kappa),'_',mat2str(theta),'_',mat2str(sigma),'_',mat2str(lambda),'.mat'];
save(datafile,'ValuesY');
phi_rY=[phi1,phi2,phi3,r_Y,kappa,theta,sigma,lambda,gamma];
save Y.mat phi_rY;

hh=r_Y;

%Needed for SECOND part
phi1_2=gamma;
phi2_2=(kappa+lambda+gamma)*0.5;
phi3_2=(2*kappa*theta)/(sigma^2);
r1_2=r_Y;
r2_2=input('Please enter the value (in %) of the spot rate in Country Y at time t=0.01: ');
r2_2=r2_2/100;
r_Y=r2_2;
ValY(1)=r_Y;
save ValY.mat ValY;

%Calculation of the Expected Exchange Rate and plot%
%Equals the expected exchange rate at evaluation point 1
ExER(1)=ER_X_Y;
i=1;
for j=0.01+t:0.01:lasttime
    i=i+1;
    ExER(i)=((1/ValuesX(3,i))/(1/ValuesY(3,i)))*ER_X_Y;
end
save ExER.mat ExER;
visu1=figure;
plot(t:0.01:lasttime,ExER,'m');hold on;

%SECOND: Calculation of the spot exchange rate at next time
%x represents time T%
integrand1=['integral1(x,',mat2str(phi1_1),',',mat2str(phi2_1),',',mat2str(phi3_1),',',mat2str(phi1_2),',',mat2str(phi2_2),',',mat2str(phi3_2),',',mat2str(r1_1),',',mat2str(r2_1),',',mat2str(r1_2),',',mat2str(r2_2),',',mat2str(ER_X_Y),')'];
integrand2=['integral2(x,',mat2str(phi1_1),',',mat2str(phi2_1),',',mat2str(phi3_1),',',mat2str(phi1_2),',',mat2str(phi2_2),',',mat2str(phi3_2),',',mat2str(r1_1),',',mat2str(r2_1),',',mat2str(r1_2),',',mat2str(r2_2),')'];
E1=quad(integrand1,1,lasttime)/quad(integrand2,1,lasttime)
ER_X_Y_at_t=[1,lasttime,E1];
save ER_X_Y_at_t.mat ER_X_Y_at_t;

clear Prt Rrt ExER
%THIRD: Expectations at next time%
load ER_X_Y_at_t.mat
%Current exchange rate
ER_X_Y=ER_X_Y_at_t(3);
%Current time%
t=ER_X_Y_at_t(1);
lasttime=ER_X_Y_at_t(2);

%COUNTRY X%
%COUNTRY X with following values
load ValX.mat
r_X=ValX(1);
kappa=ValX(2);
theta=ValX(3);
sigma=ValX(4);
lambda=ValX(5);
gamma=((kappa+lambda)^2+2*(sigma^2))^0.5;
phi1=gamma;
phi2=(kappa+lambda+gamma)*0.5;
phi3=(2*kappa*theta)/(sigma^2);

%Calculation of the term structure of COUNTRY X
%Yield converges to the spot rate as maturity nears: Tau==Null
Rrt(1)=r_X;
%Value of Zero Bond at maturity and effective interest rate
Prt(1)=1;
%i is used for counting within the array
i=1;    
for Tau=0.01:0.01:lasttime-t
    i=i+1;
    N=phi2*(exp(phi1*(Tau))-1)+phi1;    
    At=(phi1*exp(phi2*(Tau))/N)^phi3;
    Bt=(exp(phi1*(Tau))-1)/N;
    Prt(i)=At*exp(-Bt*r_X);
    if At~=0
        Rrt(i)=(-log(At)+r_X*Bt)/(Tau);
    else
        Rrt(i)=NaN;
    end
end

%Saving%
ValuesX=[t:0.01:lasttime;Rrt;Prt]; 
datafile=['X',mat2str(r_X),'_',mat2str(kappa),'_',mat2str(theta),'_',mat2str(sigma),'_',mat2str(lambda),'.mat'];
save(datafile,'ValuesX');
phi_rX=[phi1,phi2,phi3,r_X,kappa,theta,sigma,lambda,gamma];
save X.mat phi_rX;

%COUNTRY Y
%COUNTRY Y with following values
load ValY.mat
r_Y=ValY(1);
kappa=ValY(2);
theta=ValY(3);
sigma=ValY(4);
lambda=ValY(5);
gamma=((kappa+lambda)^2+2*(sigma^2))^0.5;
phi1=gamma;
phi2=(kappa+lambda+gamma)*0.5;
phi3=(2*kappa*theta)/(sigma^2);

%Calculation of the term structure of COUNTRY Y
%Yield converges to the spot rate as maturity nears: Tau==Null
Rrt(1)=r_Y;
%Value of Zero Bond at maturity and effective yield
Prt(1)=1;
%i is used for counting within the array
i=1;    
for Tau=0.01:0.01:lasttime-t
    i=i+1;
    N=phi2*(exp(phi1*(Tau))-1)+phi1;    
    At=(phi1*exp(phi2*(Tau))/N)^phi3;
    Bt=(exp(phi1*(Tau))-1)/N;
    Prt(i)=At*exp(-Bt*r_Y);
    if At~=0
        Rrt(i)=(-log(At)+r_Y*Bt)/(Tau);
    else
        Rrt(i)=NaN;
    end
end

%Saving%
ValuesY=[t:0.01:lasttime;Rrt;Prt]; 
datafile=['Y',mat2str(r_Y),'_',mat2str(kappa),'_',mat2str(theta),'_',mat2str(sigma),'_',mat2str(lambda),'.mat'];
save(datafile,'ValuesY');
phi_rY=[phi1,phi2,phi3,r_Y,kappa,theta,sigma,lambda,gamma];
save Y.mat phi_rY;

%Calculation of the Expected Exchange Rate and plot
%Equals the expected exchange rate at evaluation point 1
ExER(1)=ER_X_Y;
i=1;
for j=0.01+t:0.01:lasttime
    i=i+1;
    ExER(i)=((1/ValuesX(3,i))/(1/ValuesY(3,i)))*ER_X_Y;
end
save ExER.mat ExER;

plot(t:0.01:lasttime,ExER,'g');xlabel('Time');ylabel('Expected Exchange Rate');
title(['X: ',texlabel('r('),mat2str(ff),texlabel(')='),mat2str(gg),'; ',texlabel('r('),mat2str(t),texlabel(')='),mat2str(r2_1),'  Y: ',texlabel('r('),mat2str(ff),texlabel(')='),mat2str(hh),'; ',texlabel('r('),mat2str(t),texlabel(')='),mat2str(r2_2)],'Fontsize', 11);grid on;
legend(['at time t=',mat2str(ff)],['at time s=',mat2str(t)],0);
clear all;



